#ifndef _fencer_assets_h
#define _fencer_assets_h

#include <stdint.h>

typedef void(*AssetDestructor)(void*);

typedef uintptr_t asset_id;

// Prepare asset management.
// Other asset management functions are invalid until assets_init is called.
void assets_init();
// Clean up and shut down assets management.
void assets_clean();
// Clear all assets and reset to initial state.
void assets_reset();
// Submit an object to be managed as an asset. If the destructor is NULL, free will be used.
// Returns zero if asset could not be stored.
asset_id store_asset(void* asset, AssetDestructor destructor);
// Get an asset already submitted to the asset manager.
// Returns 0 if asset does not exist.
void* get_asset(asset_id id);
// Get an asset id, returns 0 if no such asset exists.
asset_id get_asset_id(void* asset);
// Free an asset managed by the asset manager using it's destructor, or free(..) if NULL.
void free_asset(asset_id id);

#endif // !_fencer_assets_h
