#include "tilemap.h"
#include "camera.h"
#include "program.h"
#include <stdio.h>
#include <SDL2/SDL_image.h>

#define XML(__str) (const xmlChar*)__str

struct Tilemap tilemap_load(const char* tilemap_file) {
    struct Tilemap self = {
        .dimensions = {0,0},
        .tiles = NULL
    };
    
    return self;
}

static inline
int is_path_char(int n) {
    return isalnum(n) || n == '/';
}

struct Tileset tileset_load(const char* filename) {
    char* texture_path = NULL;
    char* writer = NULL;
    char num_buffer[6];
    num_buffer[5] = '\0';
    int n = 0;
    struct Tileset self = {
        .texture = NULL,
        .tile_size = {0,0}
    };
    size_t len = 1;

    FILE* fs = fopen(filename, "r");

    do {
        n = fgetc(fs);
        if(is_path_char(n)) {
            ++len;
        }
    } while(n != ',');

    texture_path = malloc(len * sizeof(char));

    rewind(fs);
    writer = texture_path;

    do {
        n = fgetc(fs);
        if(is_path_char(n)) {
            *writer = n;
            ++writer;
        } else if(n == ',') {
            *writer = '\0';
        }
    } while(n != ',');

    self.texture = IMG_LoadTexture(g_renderer, texture_path);
    SDL_QueryTexture(self.texture, NULL, NULL, &self.texture_resolution.x, &self.texture_resolution.y);
    self.shear = floor((float)self.texture_resolution.x / self.tile_size.x);

    writer = num_buffer;

    do {
        n = fgetc(fs);
        if(isdigit(n) || n == '-') {
            *writer= n;
            ++writer;
        } else if(n == ',' || feof(fs) || n == '\n') {
            *writer = '\0';
            writer = num_buffer;
            if(self.tile_size.x == 0) {
                self.tile_size.x = atoi(num_buffer);
            } else {
                self.tile_size.y = atoi(num_buffer);
            }
        }
    } while(!feof(fs));

    fclose(fs);

    return self;
}

SDL_Rect tileset_index_to_rect(struct Tileset* self, size_t index) {
    SDL_Rect rect = {0,0, self->tile_size.x, self->tile_size.y};
    rect.x = (index % self->shear) * rect.w;
    rect.y = (index / self->shear) * rect.h;
    return rect;
}

void tilemap_render(struct Tilemap* self) {
    size_t num_tiles = self->dimensions.x * self->dimensions.y;

    SDL_Rect source_rect = {0, 0, self->tileset.tile_size.x, self->tileset.tile_size.y};
    SDL_FRect world_dest_rect = { .w = 1, .h = 1 };

    for(int i = 0; i < num_tiles; ++i) {
        source_rect = tileset_index_to_rect(&self->tileset, self->tiles[i]);

        world_dest_rect.x = (i % self->dimensions.x) * world_dest_rect.w;
        world_dest_rect.y = (float)floor((float)i / self->dimensions.y) * world_dest_rect.h;

        SDL_FRect camera_dest_rect = camera_world_to_screen_space(&g_camera, &world_dest_rect);

        SDL_RenderCopyF(g_renderer, self->tileset.texture, &source_rect, &camera_dest_rect);
    }
}